
const fs = require('fs');
const path = require('path');

/* Run cmake-js with any arguments passed to this script. */
const { execFileSync } = require('child_process');
const process = require('process');
const { platform } = require('os');
let command = "npx cmake-js " + process.argv.slice(2).join(' ');

const args = require('yargs').argv;
let buildArch = process.env.npm_config_target_arch || process.arch;
const debugBuild = args.D;

// Support being inside Electron node_modules
let photoChemistryDirRelative = '../../libphotochemistry';
if (__dirname.split(path.delimiter).pop() == 'node_modules') {
    photoChemistryDirRelative = '../' + photoChemistryDirRelative;
}

// Remove the files node_modules/.bin/rc and node_modules/.bin/rc.cmd if they exist,
// isince these interfere with the rc command used for compilation on Windows
let rcFiles = ['rc', 'rc.cmd'];
rcFiles.forEach((rcFile) => {
    let rcPath = path.join(__dirname, '../node_modules/.bin', rcFile);
    if (fs.existsSync(rcPath)) {
        fs.unlinkSync(rcPath);
    }
});

let buildDir = path.join(__dirname, photoChemistryDirRelative, 'build/node', buildArch)
// Create build dir if it doesn't exist
if (!fs.existsSync(buildDir)) {
    fs.mkdirSync(buildDir, { recursive: true });
}

console.log("buildArch is", buildArch);

// Use correct build dir
command += ` -O "${buildDir}"`
// Use vcpkg
command += ` -a ${buildArch}`
command += ` -T node-photochemistry`
if (debugBuild) {
    command += ` --CDCMAKE_BUILD_TYPE=Debug`;
 } else {
    command += ` --CDCMAKE_BUILD_TYPE=RelWithDebInfo`;
 }
command += ` --CDCMAKE_TOOLCHAIN_FILE="${process.env.VCPKG_ROOT}/scripts/buildsystems/vcpkg.cmake"`

execFileSync(command, {
    stdio: 'inherit',
    shell: true,
});

if (command.includes(" compile")  || command.includes(" build")) {
    // add 'Release' to the buildDir if on windows
    let outputDir = platform() === 'win32' ? path.join(buildDir, 'Release') : buildDir;

    // Copy the built .node file from buildDir to ./build
    let outputFile = path.join(outputDir, 'node-photochemistry.node');
    let destFile = path.join(__dirname, '../build/photochemistry.node');
    // Create dest dir if it doesn't exist
    if (!fs.existsSync(path.dirname(destFile))) {
        fs.mkdirSync(path.dirname(destFile), { recursive: true });
    }
    fs.copyFileSync(outputFile, destFile);
}

process.exit(0);
